@@ -113,8 +113,11 @@ show_error_dialog (UmAccountDialog *self,
                                          GTK_BUTTONS_CLOSE,
                                          "%s", message);
 
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                  "%s", error->message);
+        if (error != NULL) {
+                g_dbus_error_strip_remote_error (error);
+                gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                                          "%s", error->message);
+        }
 
         g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
         gtk_window_present (GTK_WINDOW (dialog));
@@ -380,8 +383,8 @@ on_permit_user_login (GObject *source,
         GError *error = NULL;
         gchar *login;
 
-        um_realm_kerberos_call_change_permitted_logins_finish (UM_REALM_KERBEROS (source),
-                                                               result, &error);
+        um_realm_kerberos_call_change_login_policy_finish (UM_REALM_KERBEROS (source),
+                                                           result, &error);
         if (error == NULL) {
 
                 /*
@@ -413,6 +416,7 @@ enterprise_permit_user_login (UmAccountDialog *self)
         gchar *login;
         const gchar *add[2];
         const gchar *remove[1];
+        GVariant *options;
 
         login = enterprise_calculate_login (self);
 
@@ -420,11 +424,13 @@ enterprise_permit_user_login (UmAccountDialog *self)
         add[1] = NULL;
         remove[0] = NULL;
 
-        um_realm_kerberos_call_change_permitted_logins (self->selected_realm,
-                                                        add, remove, "",
-                                                        self->cancellable,
-                                                        on_permit_user_login,
-                                                        g_object_ref (self));
+        options = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
+
+        um_realm_kerberos_call_change_login_policy (self->selected_realm, "",
+                                                    add, remove, options,
+                                                    self->cancellable,
+                                                    on_permit_user_login,
+                                                    g_object_ref (self));
 
         g_free (login);
 }
@@ -505,9 +511,16 @@ on_join_login (GObject *source,
 
         /* Logged in as admin successfully, use creds to join domain */
         if (error == NULL) {
-                um_realm_join (self->selected_realm,
-                               creds, self->cancellable, on_realm_joined,
-                               g_object_ref (self));
+                if (!um_realm_join_as_admin (self->selected_realm,
+                                             gtk_entry_get_text (self->join_name),
+                                             gtk_entry_get_text (self->join_password),
+                                             creds, self->cancellable, on_realm_joined,
+                                             g_object_ref (self))) {
+                        show_error_dialog (self, _("No supported way to authenticate with this domain"), NULL);
+                        g_message ("Authenticating as admin is not supported by the realm");
+                        finish_action (self);
+                }
+
                 g_bytes_unref (creds);
 
         /* Couldn't login as admin, show prompt again */
@@ -581,12 +594,15 @@ on_realm_login (GObject *source,
                         enterprise_permit_user_login (self);
 
                 /* Join the domain, try using the user's creds */
-                } else {
-                        um_realm_join (self->selected_realm,
-                                       creds,
-                                       self->cancellable,
-                                       on_realm_joined,
-                                       g_object_ref (self));
+                } else if (!um_realm_join_as_user (self->selected_realm,
+                                                   gtk_entry_get_text (self->enterprise_login),
+                                                   gtk_entry_get_text (self->enterprise_password),
+                                                   creds, self->cancellable,
+                                                   on_realm_joined,
+                                                   g_object_ref (self))) {
+
+                        /* If we can't do user auth, try to authenticate as admin */
+                        join_show_prompt (self, NULL);
                 }
 
                 g_bytes_unref (creds);
