@@ -47,6 +47,17 @@
 #define WACOM_ERASER_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.eraser"
 #define WACOM_BUTTON_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.tablet-button"
 
+static struct {
+	GnomeRRRotation  rotation;
+	GsdWacomRotation rotation_wacom;
+	const gchar     *rotation_string;
+} rotation_table[] = {
+	{ GNOME_RR_ROTATION_0,   GSD_WACOM_ROTATION_NONE, "none" },
+	{ GNOME_RR_ROTATION_90,  GSD_WACOM_ROTATION_CCW,  "ccw"  },
+	{ GNOME_RR_ROTATION_180, GSD_WACOM_ROTATION_HALF, "half" },
+	{ GNOME_RR_ROTATION_270, GSD_WACOM_ROTATION_CW,   "cw"   }
+};
+
 static WacomDeviceDatabase *db = NULL;
 
 struct GsdWacomStylusPrivate
@@ -533,7 +544,7 @@ find_output_by_edid (GnomeRRScreen *rr_screen, const gchar *vendor, const gchar
 		g_free (o_serial_s);
 
 		if (match) {
-			retval = g_object_ref (rr_outputs[i]);
+			retval = rr_outputs[i];
 			break;
 		}
 	}
@@ -558,7 +569,7 @@ find_builtin_output (GnomeRRScreen *rr_screen)
 			continue;
 
 		if (gnome_rr_output_is_laptop(rr_outputs[i])) {
-			retval = g_object_ref (rr_outputs[i]);
+			retval = rr_outputs[i];
 			break;
 		}
 	}
@@ -632,24 +643,16 @@ is_on (GnomeRROutput *output)
 }
 
 static GnomeRROutput *
-find_output_by_monitor (GdkScreen *screen,
-			int        monitor)
+find_output_by_monitor (GnomeRRScreen *rr_screen,
+			GdkScreen     *screen,
+			int            monitor)
 {
-	GError *error = NULL;
-	GnomeRRScreen *rr_screen;
 	GnomeRROutput **rr_outputs;
 	GnomeRROutput *ret;
 	guint i;
 
 	ret = NULL;
 
-	rr_screen = gnome_rr_screen_new (screen, &error);
-	if (rr_screen == NULL) {
-		g_warning ("gnome_rr_screen_new() failed: %s", error->message);
-		g_error_free (error);
-		return NULL;
-	}
-
 	rr_outputs = gnome_rr_screen_list_outputs (rr_screen);
 
 	for (i = 0; rr_outputs[i] != NULL; i++) {
@@ -669,13 +672,11 @@ find_output_by_monitor (GdkScreen *screen,
 		gnome_rr_crtc_get_position (crtc, &x, &y);
 
 		if (monitor == gdk_screen_get_monitor_at_point (screen, x, y)) {
-			ret = g_object_ref (rr_output);
+			ret = rr_output;
 			break;
 		}
 	}
 
-	g_object_unref (rr_screen);
-
 	if (ret == NULL)
 		g_warning ("No output found for monitor %d.", monitor);
 
@@ -726,33 +727,47 @@ set_display_by_output (GsdWacomDevice  *device,
 	g_free (o_serial_s);
 }
 
+static GsdWacomRotation
+get_rotation_wacom (GnomeRRRotation rotation)
+{
+        guint i;
+
+        for (i = 0; i < G_N_ELEMENTS (rotation_table); i++) {
+                if (rotation_table[i].rotation & rotation)
+                        return (rotation_table[i].rotation_wacom);
+        }
+        g_assert_not_reached ();
+}
+
 void
 gsd_wacom_device_set_display (GsdWacomDevice *device,
                               int             monitor)
 {
+	GError *error = NULL;
+	GnomeRRScreen *rr_screen;
 	GnomeRROutput *output = NULL;
 
         g_return_if_fail (GSD_IS_WACOM_DEVICE (device));
 
-	if (monitor >= 0)
-		output = find_output_by_monitor (gdk_screen_get_default (), monitor);
-	set_display_by_output (device, output);
-}
-
-static GnomeRROutput *
-find_output (GsdWacomDevice *device)
-{
-	GError *error = NULL;
-	GnomeRRScreen *rr_screen;
-	GnomeRROutput *rr_output;
-
 	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
 	if (rr_screen == NULL) {
 		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
 		g_error_free (error);
-		return NULL;
+		return;
 	}
 
+	if (monitor >= 0)
+		output = find_output_by_monitor (rr_screen, gdk_screen_get_default (), monitor);
+	set_display_by_output (device, output);
+
+	g_object_unref (rr_screen);
+}
+
+static GnomeRROutput *
+find_output (GnomeRRScreen  *rr_screen,
+	     GsdWacomDevice *device)
+{
+	GnomeRROutput *rr_output;
 	rr_output = find_output_by_display (rr_screen, device);
 
 	if (rr_output == NULL) {
@@ -767,8 +782,6 @@ find_output (GsdWacomDevice *device)
 		}
 	}
 
-	g_object_unref (rr_screen);
-
 	return rr_output;
 }
 
@@ -803,6 +816,8 @@ calculate_transformation_matrix (const GdkRectangle mapped, const GdkRectangle d
 int
 gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 {
+	GError *error = NULL;
+	GnomeRRScreen *rr_screen;
 	GnomeRROutput *rr_output;
 	GnomeRRMode *mode;
 	GnomeRRCrtc *crtc;
@@ -810,12 +825,22 @@ gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 
         g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), -1);
 
-	rr_output = find_output (device);
-	if (rr_output == NULL)
+	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
+	if (rr_screen == NULL) {
+		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+		g_error_free (error);
 		return -1;
+	}
+
+	rr_output = find_output (rr_screen, device);
+	if (rr_output == NULL) {
+		g_object_unref (rr_screen);
+		return -1;
+	}
 
 	if (!is_on (rr_output)) {
 		g_warning ("Output is not active.");
+		g_object_unref (rr_screen);
 		return -1;
 	}
 
@@ -826,7 +851,7 @@ gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 	area[2] = gnome_rr_mode_get_width (mode);
 	area[3] = gnome_rr_mode_get_height (mode);
 
-	g_object_unref (rr_output);
+	g_object_unref (rr_screen);
 
 	if (area[2] <= 0 || area[3] <= 0) {
 		g_warning ("Output has non-positive area.");
@@ -869,6 +894,32 @@ gsd_wacom_device_get_display_matrix (GsdWacomDevice *device, float matrix[NUM_EL
 	return TRUE;
 }
 
+GsdWacomRotation
+gsd_wacom_device_get_display_rotation (GsdWacomDevice *device)
+{
+	GError *error = NULL;
+	GnomeRRScreen *rr_screen;
+	GnomeRROutput *rr_output;
+	GnomeRRRotation rotation = GNOME_RR_ROTATION_0;
+
+	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
+	if (rr_screen == NULL) {
+		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+		g_error_free (error);
+		return -1;
+	}
+
+	rr_output = find_output (rr_screen, device);
+	if (rr_output) {
+		GnomeRRCrtc *crtc = gnome_rr_output_get_crtc (rr_output);
+		if (crtc)
+			rotation = gnome_rr_crtc_get_current_rotation (crtc);
+	}
+	g_object_unref (rr_screen);
+
+	return get_rotation_wacom (rotation);
+}
+
 static void
 add_stylus_to_device (GsdWacomDevice *device,
 		      const char     *settings_path,
@@ -1744,6 +1795,34 @@ gsd_wacom_device_get_button (GsdWacomDevice   *device,
 	}
 }
 
+GsdWacomRotation
+gsd_wacom_device_rotation_name_to_type (const char *rotation)
+{
+        guint i;
+
+	g_return_val_if_fail (rotation != NULL, GSD_WACOM_ROTATION_NONE);
+
+        for (i = 0; i < G_N_ELEMENTS (rotation_table); i++) {
+                if (strcmp (rotation_table[i].rotation_string, rotation) == 0)
+                        return (rotation_table[i].rotation_wacom);
+        }
+
+	return GSD_WACOM_ROTATION_NONE;
+}
+
+const char *
+gsd_wacom_device_rotation_type_to_name (GsdWacomRotation type)
+{
+        guint i;
+
+        for (i = 0; i < G_N_ELEMENTS (rotation_table); i++) {
+                if (rotation_table[i].rotation_wacom == type)
+                        return (rotation_table[i].rotation_string);
+        }
+
+	return "none";
+}
+
 GsdWacomDevice *
 gsd_wacom_device_create_fake (GsdWacomDeviceType  type,
 			      const char         *name,
